!Copyright 2017 R. CARPENTIER, P. CORDE, F. JEZEQUEL, J.-L. LAMOTTE
!
!This file is part of CADNA.
!
!    CADNA is free software: you can redistribute it and/or modify it
!    under the terms of the GNU Lesser General Public License as
!    published by the Free Software Foundation, either version 3 of the
!    License, or (at your option) any later version.
!
!    CADNA is distributed in the hope that it will be useful, but WITHOUT
!    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
!    or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General
!    Public License for more details.
!
!    You should have received a copy of the GNU Lesser General Public
!    License along with CADNA. If not, see
!    <http://www.gnu.org/licenses/>.
!

! Module de conversion




module cadna_cmplx_convert
  use ISO_C_BINDING, only : C_INT, C_FLOAT, C_DOUBLE
  use cadna_types
  use cadna_affect
  use cadna_convert
  use cadna_opp

  implicit none




  interface int
    module procedure cmplx_int_complex_single_st
  end interface

  interface int
    module procedure cmplx_int_complex_double_st
  end interface

  interface real
    module procedure cmplx_real_complex_single_st
  end interface

  interface real
    module procedure cmplx_real_complex_double_st
  end interface

  interface dble
    module procedure cmplx_dble_complex_single_st
  end interface

  interface dble
    module procedure cmplx_dble_complex_double_st
  end interface

  interface aimag
    module procedure cmplx_aimag_complex_single_st
  end interface

  interface aimag
    module procedure cmplx_aimag_complex_double_st
  end interface

  interface conjg
    module procedure cmplx_conjg_complex_single_st
  end interface

  interface conjg
    module procedure cmplx_conjg_complex_double_st
  end interface

  interface cmplx
    module procedure cmplx_cmplx_single_st
  end interface

  interface cmplx
    module procedure cmplx_cmplx_double_st
  end interface


contains



  elemental function cmplx_cmplx_single_st(a, b)
    type(single_st), intent(in) :: a, b
    type(complex_single_st) cmplx_cmplx_single_st

    cmplx_cmplx_single_st%x = a
    cmplx_cmplx_single_st%y = b
  end function cmplx_cmplx_single_st

  elemental function cmplx_cmplx_double_st(a, b)
    type(double_st), intent(in) :: a, b
    type(complex_double_st) cmplx_cmplx_double_st

    cmplx_cmplx_double_st%x = a
    cmplx_cmplx_double_st%y = b
  end function cmplx_cmplx_double_st




  elemental function cmplx_int_complex_single_st(a)
    type(complex_single_st), intent(in) :: a
    integer(C_INT) cmplx_int_complex_single_st

    cmplx_int_complex_single_st = int(a%x) 
  end function cmplx_int_complex_single_st

  elemental function cmplx_int_complex_double_st(a)
    type(complex_double_st), intent(in) :: a
    integer(C_INT) cmplx_int_complex_double_st

    cmplx_int_complex_double_st = int(a%x) 
  end function cmplx_int_complex_double_st

  elemental function cmplx_real_complex_single_st(a)
    type(complex_single_st), intent(in) :: a
    type(single_st) cmplx_real_complex_single_st

    cmplx_real_complex_single_st = a%x
  end function cmplx_real_complex_single_st

  elemental function cmplx_real_complex_double_st(a)
    type(complex_double_st), intent(in) :: a
    type(double_st) cmplx_real_complex_double_st

    cmplx_real_complex_double_st = a%x
  end function cmplx_real_complex_double_st

  elemental function cmplx_dble_complex_single_st(a)
    type(complex_single_st), intent(in) :: a
    type(double_st) cmplx_dble_complex_single_st

    cmplx_dble_complex_single_st = a%x
  end function cmplx_dble_complex_single_st

  elemental function cmplx_dble_complex_double_st(a)
    type(complex_double_st), intent(in) :: a
    type(double_st) cmplx_dble_complex_double_st

    cmplx_dble_complex_double_st = a%x
  end function cmplx_dble_complex_double_st

  elemental function cmplx_aimag_complex_single_st(a)
    type(complex_single_st), intent(in) :: a
    type(single_st) cmplx_aimag_complex_single_st

    cmplx_aimag_complex_single_st = a%y
  end function cmplx_aimag_complex_single_st

  elemental function cmplx_aimag_complex_double_st(a)
    type(complex_double_st), intent(in) :: a
    type(double_st) cmplx_aimag_complex_double_st

    cmplx_aimag_complex_double_st = a%y
  end function cmplx_aimag_complex_double_st

  elemental function cmplx_conjg_complex_single_st(a)
    type(complex_single_st), intent(in) :: a
    type(complex_single_st) cmplx_conjg_complex_single_st

    cmplx_conjg_complex_single_st = cmplx(a%x, -a%y)
  end function cmplx_conjg_complex_single_st

  elemental function cmplx_conjg_complex_double_st(a)
    type(complex_double_st), intent(in) :: a
    type(complex_double_st) cmplx_conjg_complex_double_st

    cmplx_conjg_complex_double_st = cmplx(a%x, -a%y)
  end function cmplx_conjg_complex_double_st




end module cadna_cmplx_convert

